#!/bin/sh
###
# EncFS-ROX integration - Remove decryption from directory
# Author: James Budiono - Copyright James Budiono 2010
# License: GNU GPL v3 or newer
# Note: we can use also use patched encfsctl - but I prefer to work with plain unpatched version
# v0.4 - fix symlink check and refresh rox
###

if [ -z "$1" ]; then
	Xdialog --msgbox "Run this tool from within ROX-Filer." 0 0
	exit
fi

# can't process symlink
if [ "`file \"$1\" | grep \"symbolic link\"`" ]; then
	Xdialog --title "Error" --msgbox "Symbolic link is not supported." 0 0
	exit
fi

# check - encfs exist? no ==> quit
if [ -z "`which encfs`" ]; then
	Xdialog --title "Error" --msgbox "EncFS not found - cannot continue. Please install EncFS first." 0 0
	exit
fi

# 1. prepare all the vars
ENCFS_SIG=".encfs6.xml"
ENC_DIR_WARN="This directory is encrypted"
ENC_DIR_SIG=".encrypted"

ORIG_DIR="$1"
PREFIX="${ORIG_DIR%/*}/"
DIR="${ORIG_DIR##*/}"

ENC_DIR="${PREFIX}.${DIR}"
TMP_DIR="${PREFIX}${DIR}.$RANDOM"

# ensure TMP_DIR doesn't exist.
while [ -d "${TMP_DIR}" ]; do
	TMP_DIR="${PREFIX}${DIR}.$RANDOM"
done

# checks - does storage for encrypted dir exist, with a valid signature? 
if [ ! -f "${ENC_DIR}/${ENCFS_SIG}" ]; then
	Xdialog --title "Error" --msgbox "This directory is not encrypted." 0 0
	exit
fi

# 2. Check wether it is currently mounted
MOUNTED=no
if [ -d "${ORIG_DIR}/${ENC_DIR_SIG}" ]; then 

	# 3. not mounted, so try to mount it first
	PASS1=`Xdialog --stdout --title "Decrypt" --password --inputbox "Enter password" 0 0`
	if [ -n "$PASS1" ]; then
		
		# 4. Got the password, now mount it
		echo "$PASS1" | encfs --stdin "${ENC_DIR}" "${ORIG_DIR}" -- -o nonempty
		if [ $? -eq 0 ]; then
			MOUNTED=yes
		else
			Xdialog --title "Error" --msgbox "Unable to open ${ORIG_DIR}, perhaps wrong password?" 0 0 
		fi
	else
		Xdialog --title "Cancel" --msgbox "${ORIG_DIR} not decrypted." 0 0 
	fi
else
	MOUNTED=yes
fi

# 5. if everything is ok
if [ "$MOUNTED" = "yes" ]; then
	
	# 6. make TMP_DIR
	if ! mkdir "${TMP_DIR}"; then  
		Xdialog --title "Error" --msgbox "Unable to create directory to store unencrypted files - aborting." 0 0
		exit
	fi
		
	# 7. copy everything out
	if ! cp -aT "${ORIG_DIR}" "${TMP_DIR}"; then
		rm -rf "${TMP_DIR}"
		Xdialog --title "Error" --msgbox "Unable to copy unencrypted files. Perhaps disk full? - aborting" 0 0
		exit
	fi

	# 7.1 sanity check - the number of entries in original and encrypted must be the same
	SRC_COUNT=`ls -lR "${TMP_DIR}" | wc -l`
	TGT_COUNT=`ls -lR "${ORIG_DIR}" | wc -l`
	if [ ! SRC_COUNT -eq TGT_COUNT ]; then
		rm -rf "${TMP_DIR}"
		Xdialog --title "Error" --msgbox "Some unencrypted files cannot be copied - aborting." 0 0
		exit
	fi 
	
	# 8. Unmount encfs directory
	if ! fusermount -u "${ORIG_DIR}"; then
		rm -rf "${TMP_DIR}"
		Xdialog --title "Error" --msgbox "Unable to close the encrypted directory - aborting" 0 0	
		exit
	fi
		
	# 9. delete the mountpoint and storage
	rm -rf "${ENC_DIR}" "${ORIG_DIR}" 
	#If this fails, there is nothing we can at this point
	
	# 10. rename TMP_DIR to ORIG_DIR
	if mv "${TMP_DIR}" "${ORIG_DIR}"; then
		rox -x "${PREFIX}"
		Xdialog --title "Success!" --msgbox "Directory ${ORIG_DIR} is now decrypted." 0 0
	else
		Xdialog --title "WARNING" --msgbox \
			"Decryption is successful, 
			but your files cannot be moved to its original location.
			Your encrypted files is now available in ${TMP_DIR}." 0 0
	fi
	
fi

