#!/bin/sh
###
# EncFS-ROX integration - Add encryption to directory
# Author: James Budiono - Copyright James Budiono 2010
# License: GNU GPL v3 or newer
# v0.4 - fix symlink check and refresh rox
###

if [ -z "$1" ]; then
	Xdialog --msgbox "Run this tool from within ROX-Filer." 0 0
	exit
fi

# can't process symlink
if [ "`file \"$1\" | grep \"symbolic link\"`" ]; then
	Xdialog --title "Error" --msgbox "Symbolic link is not supported." 0 0
	exit
fi

# check - encfs exist? no ==> quit
if [ -z "`which encfs`" ]; then
	Xdialog --title "Error" --msgbox "EncFS not found - cannot continue. Please install EncFS first." 0 0
	exit
fi

# 1. prepare all the vars
ENCFS_SIG=".encfs6.xml"
ENC_DIR_WARN="This directory is encrypted"
ENC_DIR_SIG=".encrypted"

ORIG_DIR="$1"
PREFIX="${ORIG_DIR%/*}/"
DIR="${ORIG_DIR##*/}"

ENC_DIR="${PREFIX}.${DIR}"
TMP_DIR="${PREFIX}${DIR}.$RANDOM"

# ensure TMP_DIR doesn't exist.
while [ -d "${TMP_DIR}" ]; do
	TMP_DIR="${PREFIX}${DIR}.$RANDOM"
done

# checks - storage for encrypted dir exist? ==> yes - assume already encrypted - quit
if [ -d "${ENC_DIR}" ]; then
	Xdialog --title "Error" --msgbox "Directory is already encrypted (${ENC_DIR} already exist)." 0 0
	exit
fi

# checks - are we trying to encrypt an encrypted storage directory?
if [ -f "${ORIG_DIR}/${ENCFS_SIG}" ]; then
	Xdialog --title "Error" --msgbox "Directory is already encrypted (${ORIG_DIR} contains encrypted files)." 0 0
	exit
fi

# 2. Ask for encryption type
#ENC_TYPE=" "  #either (space)-standard or p-aranoid
ENC_TYPE=`Xdialog --stdout --title "Encrypt" --radiolist "Please choose your preferred encryption strength" 0 0 3 Standard "160-bit Blowfish" on Extra "256-bit AES" off `
case "$ENC_TYPE" in
	Standard) ENC_TYPE=" "
	;;
	Extra) ENC_TYPE="p"
	;;
	*) exit
	;;
esac

# 3. get the password
PASS1="x"
while [ "$PASS1" != "$PASS2" ]; do
	PASS1=`Xdialog --stdout --title "Encrypt" --password --inputbox "Type new password" 0 0`
	if [ ! $? -eq 0 ]; then exit; fi
	if [ -z "$PASS1" ]; then 
		Xdialog --msgbox "Password cannot be empty. Please re-enter." 0 0
		PASS1="x"
		continue;
	fi
	PASS2=`Xdialog --stdout --title "Encrypt" --password --inputbox "Confirm new password" 0 0`
done
if ! Xdialog --title "WARNING" --ok-label OK --cancel-label Cancel --yesno \
		"Please ensure you can remember the password.
		Please keep it in a safe place and don't lose it.
		If you forget it, you cannot access your files EVER again.
		If this is fine, click OK to continue, otherwise click Cancel" 0 0; then exit
fi

# 4. rename the original directory to a tmp-name ($TMP_DIR now contains original files)
mv "${ORIG_DIR}" "${TMP_DIR}"
if [ -d "${ORIG_DIR}" ]; then # check - original still exist? something is very wrong
	Xdialog --title "Error" --msgbox "Unable to rename original directory ${ORIG_DIR}" 0 0
	exit
fi

# 5. create new directories - the encrypted storage dir, and the (new) original directory
mkdir "${ENC_DIR}" "${ORIG_DIR}"
if [ ! $? -eq 0 ]; then  # check - mkdir failed? yes ==> clean up and quit
	rmdir "${ENC_DIR}" "${ORIG_DIR}"
	mv "${TMP_DIR}" "${ORIG_DIR}" 
	Xdialog --title "Error" --msgbox "Unable to create storage directory for encrypted files ${ENC_DIR}" 0 0
	exit
fi
touch "${ORIG_DIR}/${ENC_DIR_WARN}"
mkdir "${ORIG_DIR}/${ENC_DIR_SIG}"

# 6. The objective of the whole script - create the encrypted dir
cat <<EOF | encfs --stdin "${ENC_DIR}" "${ORIG_DIR}" -- -o nonempty
$ENC_TYPE
$PASS1
$PASS1
EOF

# check - encfs failed? clean up amd quit
if [ ! $? -eq 0 ]; then
	fusermount -u "${ORIG_DIR}"
	rm -rf "${ENC_DIR}" "${ORIG_DIR}"
	mv "${TMP_DIR}" "${ORIG_DIR}" 
	Xdialog --title "Error" --msgbox "Unable to initialise encryption - aborting." 0 0
	exit
fi

# 7. copy all the files
cp -aT "${TMP_DIR}" "${ORIG_DIR}"
if [ ! $? -eq 0 ]; then # check - copy failed? clean up
	fusermount -u "${ORIG_DIR}"
	rm -rf "${ENC_DIR}" "${ORIG_DIR}"
	mv "${TMP_DIR}" "${ORIG_DIR}" 
	Xdialog --title "Error" --msgbox "Unable to copy files to encrypted storage directory - aborting." 0 0
	exit
fi

# 8. sanity check - the number of entries in original and encrypted must be the same
SRC_COUNT=`ls -lR "${TMP_DIR}" | wc -l`
TGT_COUNT=`ls -lR "${ORIG_DIR}" | wc -l`
if [ ! SRC_COUNT -eq TGT_COUNT ]; then
	fusermount -u "${ORIG_DIR}"
	rm -rf "${ENC_DIR}" "${ORIG_DIR}"
	mv "${TMP_DIR}" "${ORIG_DIR}" 
	Xdialog --title "Error" --msgbox "Some files cannot be copied to encrypted storage directory - aborting." 0 0
	exit
fi 

# 9. remove the original unencrypted files (from TMP_DIR)
rm -rf "${TMP_DIR}" 
if [ ! $? -eq 0 ]; then # if failed, we can't do anything at this stage - just warn user
	Xdialog --title "WARNING" --msgbox "Unable to delete original unencrypted files in ${TMP_DIR}" 0 0
	exit
fi

# 9. We're done.
rox -x "${PREFIX}"
Xdialog --title "Success!" --msgbox "Directory ${ORIG_DIR} is now encrypted." 0 0
